#ifndef _UDM_COMMON_H
#define _UDM_COMMON_H

#include "udm_os.h"
#include <time.h>
#include <netinet/in.h>
#include <stdio.h> /* for FILE etc. */
#include "udm_crc32.h"

/* Some constants */
#define UDMSTRSIZ	1024*5

#define UDM_MAXPARSER	256	/* FIXME */

/* Some constants */
#define UDM_LANGPERDOC		16
#define UDM_URLSIZE		127
#define UDM_USER_AGENT		"UdmSearch"


/* Locking consts */
#define UDM_LOCK		1
#define UDM_UNLOCK		2
#define UDM_LOCK_USER		0
#define UDM_LOCK_MEXUSER	15
#define UDM_LOCK_ROBOTS		16
#define UDM_LOCK_QUERY		17
#define UDM_LOCK_TARGET		18
#define UDM_LOCK_RESOLVE	19
#define UDM_LOCK_MAX		20


/* storage types */
#define UDM_DBMODE_SINGLE	0
#define UDM_DBMODE_MULTI	1
#define UDM_DBMODE_SINGLE_CRC	2
#define UDM_DBMODE_WORD2URL	3
#define UDM_DBMODE_MULTI_CRC	4
#define UDM_DBMODE_CACHE	5


/* database open modes */
#define UDM_OPEN_MODE_READ	0
#define UDM_OPEN_MODE_WRITE	1


/* search modes */
#define UDM_ORD_RATE		0
#define UDM_ORD_DATE		1
#define UDM_MODE_ALL		0
#define UDM_MODE_ANY		1
#define UDM_MODE_BOOL		2

/* search cache */
#define UDM_CACHE_ENABLED	1
#define UDM_CACHE_DISABLED	0

/* Indexer return codes */
#define IND_UNKNOWN		0
#define IND_OK			1
#define IND_ERROR		2
#define IND_NO_TARGET		3
#define IND_TERMINATED		4


/* Flags for indexing */
#define UDM_FLAG_REINDEX	1
#define UDM_FLAG_EXP_FIRST	2
#define UDM_FLAG_ADD_SERV	4
#define UDM_FLAG_MARK		8
#define UDM_FLAG_SPELL		16
#define UDM_FLAG_INSERT		32
#define UDM_FLAG_SORT_HOPS	64
#define UDM_FLAG_INIT		128
#define UDM_FLAG_SKIP_LOCKING	256

/* Tracking modes */
#define UDM_TRACK_DISABLED	0
#define UDM_TRACK_QUERIES	1
#define UDM_TRACK_WORDS		2


/* URLFile actions */
#define UDM_URL_FILE_REINDEX	1
#define UDM_URL_FILE_CLEAR	2
#define UDM_URL_FILE_INSERT	3
#define UDM_URL_FILE_PARSE	4

/* Ispell modes */
#define UDM_ISPELL_MODE_TEXT	0
#define UDM_ISPELL_MODE_DB	1

/* UdmSearch types */


typedef struct udm_crcword_struct {
	int	weight;
	int	url_id;
	int	wrd_id;
} UDM_CRCWORD;


/* StopList unit */
typedef struct udm_stopword_struct {
	char *word;
	char lang[3];
} UDM_STOPWORD;


/* Word list unit */
typedef struct udm_word_struct {
	int count;
	char *word;
} UDM_WORD;


/* Language quesser structure for one language   */
/* We could use the type above instead of course */
typedef struct udm_lang_struct {
	int  count;
	char lang[3];
} UDM_LANG;


/* Structure to store server parameters */
typedef struct udm_server_struct {
	/* General items */
	char *url;
	char *charset;
	char *category;
	char *tag;
	char lang[3];
	int  period;

	/* Auth */
	char *basic_auth;
	char *user;
	char *passwd;

	/* Proxy stuff */
	char *proxy;
	char *proxy_basic_auth;
	int  proxy_port;

        /* HTDB */
	char *htdb_list;
	char *htdb_doc;

	/* Mirror stuff */
	int  use_mirror;
	char *mirror_root;
	char *mirror_headers;

	/* Network stuff */
	int  net_errors;
	int  max_net_errors;
	int  net_error_delay_time;
	int  read_timeout;

	/* Spider & indexer stuff */
	int  index;
	int  follow;
	int  maxhops;
	int  deletebad;
	int  use_robots;
	int  use_clones;
	int  delete_no_server;

	/* Words stuff */
	int  bodyweight;
	int  titleweight;	
	int  urlweight;
	int  urlhostweight;
	int  urlpathweight;
	int  urlfileweight;
	int  descweight;
	int  keywordweight;
	int  correct_factor;
	int  incorrect_factor;
	int  number_factor;
	int  alnum_factor;
	int  min_word_length;
	int  max_word_length;

	/* MP3 extensions */
	int check_mp3_tag;
	int check_only_mp3_tag;
} UDM_SERVER;


/* All components in this structure correspondent */
/* to fields in the "url" table                   */

typedef struct udm_url_struct {
	int	url_id;
	int	status;
	int	size;
	int	rating;
	int	order; /* For search result */
	int	referrer;
	int	tag;
	int	hops;
	int	indexed; /* for own backend */
	char	*url;
	char	*content_type;	
	char	*title;
	char	*keywords;
	char	*description;
	char 	*text;
	char	*category;	
	time_t	last_mod_time;
	time_t	last_index_time;
	time_t  next_index_time;
	udmcrc32_t	crc32;

} UDM_DOCUMENT;

typedef struct udm_conn_struct {
        int status;
        int connected;
        int err;
        int retry;
        int conn_fd;
        int port;
        int timeout;
        char *hostname;
        FILE *in, *out;
        struct sockaddr_in sin;
        int buf_len;
        int buf_len_total;
        int len;
        char *buf;
        struct udm_conn_struct *connp;
        struct udm_indexer_struct *indexer;
} UDM_CONN;

/* Aliases item structure */
typedef struct udm_alias_struct {
	char *find;
	char *replace;
} UDM_ALIAS;


/* All links are stored in the cache of this structure */
/* before actual INSERT into database                  */
typedef struct udm_href_struct {
	char *href;
	int referrer;
	int hops;
	int stored;
} UDM_HREF;


/* External Parsers */
typedef struct udm_parser_struct{
        char *from_mime;
	char *to_mime;
        char *parser;
	char **argv;
	int fileposition;	
	char * charset;
} UDM_PARSER;

/* Resolve stuff */
typedef struct udm_host_addr_struct {
	char *hostname;
	struct in_addr addr;
	int net_errors;
	time_t last_used;
}UDM_HOST_ADDR;

typedef struct spell_struct {
	char * word;
	char flag[10];
	char lang[3];
	int  code;
} UDM_SPELL;


typedef struct udm_robots_struct {
	char *hostinfo;
	char *path;
} UDM_ROBOT;



/* Config file */
typedef struct udm_config_struct {
	int errcode;
	char errstr[UDMSTRSIZ];
	int local_charset;
	int force1251;
	int max_doc_size;
	char user_agent[UDMSTRSIZ];
	char extra_headers[UDMSTRSIZ];
	char * url_file_name;
	int nservers;
	int mservers;
	UDM_SERVER * Server;
	UDM_SERVER * csrv;
	int naliases;
	int maliases;
	UDM_ALIAS * Alias;
	int mhrefs;
	int nhrefs;
	int shrefs;
	int dhrefs;
	UDM_HREF * Href;
	int have_targets;
	int nrows;
	int currow;
	void * Filter;
	int nfilters;
	int mfilters;
	int nmimes;
	int mmimes;
	void * Mime;
	int nparsers;
	UDM_PARSER parsers[UDM_MAXPARSER];
	int nhost_addr;
	int mhost_addr;
	UDM_HOST_ADDR * host_addr;
	int naffixes;
	int maffixes;
	void * Affix;
	int nspell;
	int mspell;
	UDM_SPELL * Spell;
	int nstoplist;
	UDM_STOPWORD * stoplist;
	int ispell_mode;
	UDM_ROBOT * Robots;
	char * DBHost;
	char * DBName;
	char * DBUser;
	char * DBPass;
	int    DBPort;
	int    DBType;
	int    DBMode;
	int    DBUseLock;
	char catstr[UDMSTRSIZ];
	char langstr[UDMSTRSIZ];
	char tagstr[UDMSTRSIZ];
	char statusstr[UDMSTRSIZ];
	char urlstr[UDMSTRSIZ];
	char timestr[UDMSTRSIZ];
	char category[UDMSTRSIZ];

	char srv_tables[UDMSTRSIZ];

	/* for logger */
	int logFacility;
	int logLevel;
	int is_log_open;
	FILE *logFD;

	
	/* For bult-in database */
	char * ul;
	char * ttag;
	int n_urls;
	UDM_DOCUMENT * URLs;
	
	/* For GetDocInfo cache */
	void * urlres;
	
	void (*ThreadInfo)(int handle,char * state,char * str);
	void (*StatInfo)(int handle, int code, int expired, int total, char* str);
	void (*RefInfo)(int code,char *url, char *ref);
	void (*LockProc)(int command, int type);

} UDM_ENV;


typedef struct udm_url {
	char schema[UDM_URLSIZE];
	char specific[UDM_URLSIZE];
	char hostinfo[UDM_URLSIZE];
	char auth[UDM_URLSIZE];
	char hostname[UDM_URLSIZE];
	char path[UDM_URLSIZE];
	char filename[UDM_URLSIZE];
	char anchor[UDM_URLSIZE];
	int  port;
} UDM_URL;

typedef struct udm_tag {
	char *tag;
	char *href;
	char *src;
	char *content;
	char *value;
	char *selected;
	char *name;
	char *type;
} UDM_TAG;

typedef struct udm_search_wrd_struct{
	int  url_id;
	short  count;
	short  weight;
} UDM_SEARCHWORD;

typedef struct udm_category_struct {
	int   rec_id;
	char  path[128];
	char  link[128];
	char  name[128];
} UDM_CATEGORY;


/* Indexer */
typedef struct udm_indexer_struct{
	int handle;		/* Indexer handle for multithreaded version */
	int mwords;		/* Number of memory allocated for words     */
	int nwords;		/* Real number of words in list             */
	int swords;		/* Number of words in sorted list           */
	UDM_WORD *Word;		/* Word list  itself                        */
	int DBError;	
	int action;		/* Callback function may request action     */
	char state[128];	/* Currently unused                         */
        int buf_size;		/* Currently unused                         */
	char *buf;		/* Buffer for document                      */
	void * db;              /* Database handle                          */
	UDM_LANG lang[UDM_LANGPERDOC]; /* language guesser variable         */
	UDM_CONN *connp;
	UDM_ENV * Conf;
	char wordinfo[1024];
	int  charset;
	int  search_mode;
	int  sort_order;
	int  page_number;
	int  page_size;
	unsigned int group_mask;
	int  track_mode;
	UDM_SPELL t_Spell;
	int read_timeout;
	int maxsize;
	int cache_mode;
	int crcdict[257];
	int work_time;
	int total_found;
} UDM_AGENT;

typedef struct {
	int first;
	int last;
	int total_found;
	int num_rows;
	UDM_DOCUMENT * Doc;
} UDM_RESULT;

/* Boolean search constants and types */
#define UDM_MAXSTACK 100
#define UDM_STACK_LEFT  0
#define UDM_STACK_RIGHT 1
#define UDM_STACK_BOT   2
#define UDM_STACK_OR    3
#define UDM_STACK_AND   4
#define UDM_STACK_NOT   5
#define UDM_STACK_WORD  200

typedef struct {
	int ncstack;
	int cstack[UDM_MAXSTACK];
	int nastack;
	int astack[UDM_MAXSTACK];
} UDM_BOOLSTACK;

typedef struct {
	int cmd;
	int arg;
} UDM_STACK_ITEM;


#endif
